use std::cell::RefCell;
use std::rc::Rc;

use super::create_vram_bg;

use super::main_tab::SharedMainTab;
use super::MainWindowRef;
use super::{file_tab::{FileTab, SharedFileTab, MutexTIMManager}, main_tab::MainTab};

pub struct GUIElements {
    pub file_tab: SharedFileTab,
    pub main_tab: SharedMainTab,
}

impl GUIElements {
    pub fn new(main_window: MainWindowRef, tim_manager: MutexTIMManager) -> Result<GUIElements, slint::PlatformError> {
        main_window.borrow().set_main_tab_vram_bg(GUIElements::create_bg()?);

        let main_tab = MainTab::new(main_window.clone(), tim_manager.clone());
        let file_tab = FileTab::new(main_window.clone(), main_tab.clone(), tim_manager);
        
        Ok(GUIElements{file_tab, main_tab})
    }

    fn create_bg() -> Result<slint::Image, slint::PlatformError> {
        Ok(slint::Image::from_rgba8_premultiplied(create_vram_bg(320, 256).ok_or(slint::PlatformError::Other("Failed creating VRAM background image".to_string()))?))
    }
}