// Prevent console window in addition to Slint window in Windows release builds when, e.g., starting the app via file manager. Ignored on other platforms.
#![cfg_attr(not(debug_assertions), windows_subsystem = "windows")]

mod gui;
use gui::{GUIElements, VRAM_WIDTH, VRAM_HEIGHT, display_information};
use rfd::FileDialog;
use std::{cell::RefCell, rc::Rc, sync::Mutex};
use slint::SharedString;
use tim_tool::logic::{tim::types::Encoding, TIMManager};
use tool_helper::Error;

slint::include_modules!();

struct Application {
    main_window:  Rc<RefCell<MainWindow>>,
    gui_elements: Rc<RefCell<GUIElements>>,
    tim_manager:  Rc<Mutex<TIMManager>>,
}

impl Application {
    fn new() -> Result<Application, slint::PlatformError> {
        let main_window = Rc::new(RefCell::new(MainWindow::new()?));
        let tim_manager = Rc::new(Mutex::new(TIMManager::new()));

        Ok(Application{
            main_window:  main_window.clone(),
            gui_elements: Rc::new(RefCell::new(GUIElements::new(main_window.clone(), tim_manager.clone())?)),
            tim_manager:  tim_manager,
        })
    }

    fn setup(&self) {
        self.setup_about_tab();
    }

    fn run(&self) -> Result<(), slint::PlatformError> {
        let main_window = self.main_window.borrow();
        main_window.run()
    }
        
    fn setup_about_tab(&self) {
        const VERSION: &str = env!("CARGO_PKG_VERSION");
    
        self.main_window.borrow().invoke_set_version(SharedString::from(VERSION));
    }
}

fn main() -> Result<(), slint::PlatformError> {
    let application = Application::new()?;

    application.setup();
    application.run()
}